@@ -6,7 +6,7 @@
 #' requirement for repeated measurements on all research subjects, the analysis assumes
 #' that multiple observations are made. 
 #' 
-#' This function performs analyses as described in Collyer and Adams (in press)
+#' This function performs analyses as described in Collyer and Adams (2024)
 #'  to assess systematic and random components of 
 #' measurement error (ME).  It basically performs ANOVA with RRPP,
 #' but with different restricted randomization strategies.  The reliability of research subject variation 
@@ -22,25 +22,31 @@
 #' including the inverse of the random component of ME and the systematic
 #' component of ME.  The multivariate test is a form of multivariate ANOVA (MANOVA), using
 #' RRPP to generate sampling distributions of the major eigenvalue (Roy's maximum root).
-#' Intraclass correlation coefficients (ICC) can also be calculated (using \code{\link{ICCstats}}), 
+#' Likelihood-ratio tests can also be performed using \code{\link{lr_test}}.
+#' 
+#' Intraclass correlation coefficients (ICC) can also be 
+#' calculated (using \code{\link{ICCstats}}), 
 #' both based on dispersion of values and 
-#' covariance matrices, as descriptive statistics.  Details are provided in \code{\link{ICCstats}}.
+#' covariance matrices, as descriptive statistics.  
+#' Details are provided in \code{\link{ICCstats}}.
 #'
-#' Examples TBD
 #' 
-#' 
-#' @param Y A matrix (n x p) of data for n observations and p variables.
-#' @param subjects A vector or factor of research subjects (each subject should occur twice or more).  
-#' The length of the vector must equal the number of observations and will be coerced into a factor.
-#' @param replicates A vector or factor for replicate measurements for research subjects.  
-#' The length of the vector must equal the number of observations and will be coerced into a factor.
-#' @param groups An optional vector, coercible to factor, to be included in the linear model
-#' (as an interaction with replicates)..
-#' This would be of interest if one were concerned with systematic ME occurring perhaps differently among 
-#' certain strata within the data.  For example, systematic ME because of an observer bias might
-#' only be observed with females or males.  
-#' @param data An optional data frame, either of class \code{\link{data.frame}} or
-#' class \code{\link{rrpp.data.frame}}.
+#' @param data A required data frame, either of class \code{\link{data.frame}} or
+#' class \code{\link{rrpp.data.frame}}.  This function cannot be used without a
+#' data frame.  All arguments for data and variables are names that must exist in the data frame.
+#' @param Y A name for a matrix (n x p) of data for n observations and p variables that can be found
+#' in the data frame.  For example, Y = "morphData".
+#' @param subjects A name for a vector or factor of research subjects, found within the data frame
+#'(each subject should occur twice or more).  The length of the vector in the data frame must equal the 
+#' number of observations and will be coerced into a factor.  For example, subjects = "ID".
+#' @param replicates A name for a vector or factor for replicate measurements for research subjects, found
+#' within the data frame.  The length of the vector  in the data frame must equal the number of observations and will 
+#' be coerced into a factor.  For example, replicates = "Rep".
+#' @param groups An optional name for a vector in the data frame, coercible to factor, to be included 
+#' in the linear model (as an interaction with replicates).  This would be of interest if one 
+#' were concerned with systematic ME occurring perhaps differently among certain strata within the data.  
+#' For example, systematic ME because of an observer bias might only be observed with females or males,
+#' in which case the argument might be: groups = "Sex".
 #' @param iter Number of iterations for significance testing
 #' @param seed An optional argument for setting the seed for random 
 #' permutations of the resampling procedure.
@@ -57,11 +63,11 @@
 #' This might be helpful for relative eigenanalysis, and if p > n, 
 #' in which case inverting singular covariance matrices would not be possible.
 #' @param tol A value indicating the magnitude below which 
-#' components should be omitted., if use.PCs is TRUE. (Components are omitted if their 
+#' components should be omitted, if use.PCs is TRUE. (Components are omitted if their 
 #' standard deviations are less than or equal to tol times the 
 #' standard deviation of the first component.)  See \code{\link{ordinate}} for more details.
 #' @param Parallel The same argument as in \code{\link{lm.rrpp}} to govern parallel processing (
-#' either a logical vale -- TRUE or FALSE -- or the number of threaded cores to use).  See \code{\link{lm.rrpp}} 
+#' either a logical value -- TRUE or FALSE -- or the number of threaded cores to use).  See \code{\link{lm.rrpp}} 
 #' for additional details.
 #' @param turbo Logical value for whether to suppress coefficient estimation in RRPP iteration,
 #' thus turbo-charging RRPP.
@@ -86,11 +92,12 @@
 #'  that yield orthogonal eigenvectors.}
 
 #' @references Collyer, M.L. and D.C. Adams.  2024. Interrogating Random and Systematic Measurement Error 
-#' in Morphometric Data. Evolutionary Biology.
+#' in Morphometric Data. Evolutionary Biology, 51, 179–20.
 #' @references Bookstein, F.L., & Mitteroecker, P. (2014). Comparing covariance matrices by relative eigenanalysis, 
-#' with applications to organismal biology. Evolutionary biology, 41(2), 336-350.
+#' with applications to organismal biology. Evolutionary Biology, 41(2), 336-350.
 
-#' @seealso \code{\link{lm.rrpp.ws}}, \code{\link{manova.update}}
+#' @seealso \code{\link{lm.rrpp.ws}}, \code{\link{manova.update}}, 
+#' \code{\link{lr_test}}
 
 #' @examples
 #' \dontrun{
@@ -132,11 +139,11 @@
 #' focusMEonSubjects(P, subjects = 18:20, shadow = TRUE)
 #' }
 #' 
-measurement.error <- function(Y, 
+measurement.error <- function(data, 
+                              Y, 
                               subjects, 
                               replicates, 
                               groups = NULL,
-                              data,
                               iter = 999, 
                               seed = NULL,
                               multivariate = FALSE,
